/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/**
 * @name idx.trees
 * @class Provides an a11y fix to dijit.Tree for high-contrast mode that helps the tree
 *        determine that clicking the text node replacement for the "expando" image 
 *        behave the same as clicking the image.  Include this extension to enable the fix.
 *        This is included from "idx/ext" module as well.
 */
define(["dojo/_base/lang","idx/main","dojo/dom","dijit/Tree"],
		function (dLang,iMain,dDom,dTree) {
	var iTrees = dLang.getObject("trees", true, iMain);
	
	// get the tree prototype
	var baseProto = dTree.prototype;
    
	// 
	// Get the base function so we can call them from our override
	//
	var baseIsExpando  = baseProto.isExpandoNode;
	
	baseProto.isExpandoNode = function(node, widget) {
		var baseResult = baseIsExpando.call(this,node,widget);
		if (baseResult) return true;
		if (! widget.expandoNodeText) return false;
		return dDom.isDescendant(node, widget.expandoNodeText);
	};
	
	return iTrees;
});


